function MarkerView(fname,mode)
   
MODE = upper(mode);             % upper case pacer mode

%  codes for event types
E_VP    = 1;                    % ventricle pace
E_VS    = E_VP+1;               % ventricle sense
E_VES   = E_VS+1;               % ventricle extrasystole
E_VRS   = E_VES+1;              % ventricle refractory sense
E_VSBLK = E_VRS+1;              % blanked (undetected) ventricle event
E_AP    = E_VSBLK+1;            % atrial pace
E_AS    = E_AP+1;               % atrial sense
E_ARS   = E_AS+1;               % atrial refractory sense
E_ASFFP = E_ARS+1;              % atrial sense in far field protection window
E_ASPMT = E_ASFFP+1;            % atrial sense in PVARP (PMT protection window)
E_ASBLK = E_ASPMT+1;            % blanked atrial event

num_types = E_ASBLK+1;          % number of event types

% load marker file
[t, event, AESC, VESC, AB, ARP, FFP, PVARP, SW, VB, VRP, UTI, WKB, AVD ...
    LastAA, LastAV, LastVV, LastVA, stat] ...
    = textread(fname,'%f %d %f %f %f %f %f %f %f %f %f %f %d %f %f %f %f %f %f');
event = event+1;                % resort event type to start from 1

for i = 1:num_types,            % for all event types,
    indx{i} = find(event == i); % find the corresponding event indexes
end
iAB = find((event>=E_AP & event<=E_ASPMT) | event==E_VP); % events w/ AB
iVB = find((event>=E_VP & event<=E_VRS) | event==E_AP);   % events w/ VB
iARP = find(event==E_AP | event==E_ASPMT | event==E_AS);  % events w/ ARP
iVRP = find(event==E_VP | event==E_VES | event==E_VS);    % events w/ VRP
iFFP = find(event==E_VP | event==E_VES | event==E_VS);    % events w/ FFP
iPVARP = find(event==E_VP | event==E_VES);                % events w/ PVARP
iUTI = find(event==E_VP | event==E_VES | event==E_VS);    % events w/ UTI
iAx = find(event==E_AP | event==E_AS);                    % AP and AS event
iSW = find(event==E_AP & SW>0);                % find all AP events triggering SW
iSWVS=find(event==E_VS & SW>0);                % find all safety window VS events
iWKBa = find((event==E_AP | event==E_AS) & WKB>0);        % find Ax starting WKB 
iWKBv = find(event==E_VP & WKB>0);                        % find Vp w/ WKB 

% find all conducted Vs events preceeded by AsPmt events
iVs = indx{E_VS};                              % indexes of VS events
iVsPmt = [];                                   % init set of VS after ASPMT
for i = 1:length(iVs),                         % for all VS events
    s = find(iAVC < iVs(i));                   % search preceeding As or AsPmt events
    if ~isempty(s) & event(iAVC(s(end)))==E_ASPMT,
        iVsPmt = [iVsPmt; iVs(i)];             % add detected Vs following AsPmt 
    end
end

% define begins and ends of all Ap timers (iAP1,iAP2) and Vp timers (iVP1, iVP2)
iAP1 = [];                                     % init set of Ap timer begins
iVP1 = [];                                     % init set of Vp timer begins
iAP2 = [];                                     % init set of Ap timer ends
iVP2 = [];                                     % init set of Vp timer ends
if strcmp(MODE,'DDI') | strcmp(MODE,'VVI') | strcmp(MODE,'DVI') | strcmp(MODE,'VOO'), % V-based 
    iVP1 = setdiff(iFFP, iSWVS);               % Vp timer begins w/ Vp, Vs, VES (excl. SW_VS)
    iVpEnd = iFFP;                             % Vp timer ends w/ VP, VS, VES
    if strcmp(MODE,'DDI') | strcmp(MODE,'DVI'),% V-based timing w/ Ap (DDI,DVI)
        iAP1 = iVP1;                           % Ap timer begins w/ Vp, Vs, VES (excl. SW_VS)
        iApEnd = union(iAx, iFFP);             % Ap timer ends w/ Ap, As, Vp, Vs, VES
    end
else                                           % A-based timing (DDD,AAI,AOO)
    iAP1 = iAx;                                % Ap timer begins w/ Ap, As
    iAP1 = union(iAP1, indx{E_VES});           % VES also starts Ap timer (BI)
    iAP1 = union(iAP1, iVsPmt);                % Vs after AsPmt also starts Ap timer (VA)
    iAP1 = union(iAP1, iWKBv);                 % WKB Vp also starts Ap timer (VA)
    iApEnd = iAP1;                             % Ap timer ends w/ Ap, As, VES, VsPmt
    if strcmp(MODE,'DDD'),                     % A-based timing w/ Vp (DDD)
        iVP1 = iAx;                            % Vp timer begins w/ Ap, As
        iVpEnd = union(iVP1, iFFP);            % Vp timer ends w/ Vp, Vs, and VES
    end
end
% find complete Ap timers (end w/ iAP2) and Vp timers (end w/ iVP2)
if MODE(1)=='D' | MODE(1)=='A',                % modes w/ Ap (DDD,DDI,AAI,AOO)
	for i = 1:length(iAP1),                    % for all Ap timers
        s = find(iApEnd > iAP1(i));            % search following ApEnd events
        if ~isempty(s),
            iAP2 = [iAP2; iApEnd(s(1))];       % add detected ApEnd event
        end
	end
	iAP1 = iAP1(1:length(iAP2));               % update begins of complete AP timers
end
if MODE(1)=='D' | MODE(1)=='V',                % modes w/ Vp (DDD,DDI,VVI,VOO,DVI)
	for i = 1:length(iVP1),                    % for all Vp timers
        s = find(iVpEnd > iVP1(i));            % search following VpEnd events
        if ~isempty(s),
            iVP2 = [iVP2; iVpEnd(s(1))];       % add detected VpEnd event
        end
	end
	iVP1 = iVP1(1:length(iVP2));               % update begins of complete VP timers
end

figure(1); clf;
axis([0 t(end) -12 12]);
xlabel('time(s)');
hold on
% plot event markers
plotevent(t, indx{E_AP}, 'P', 9, stat, 'r-o');
plotevent(t, indx{E_AS}, 'S', 9, stat, 'b-^');
plotevent(t, indx{E_ASBLK}, 'S', 6, stat, 'b-.x');
plotevent(t, indx{E_ARS}, 'S', 6, stat, 'b-x');
plotevent(t, indx{E_ASFFP}, 'S', 7, stat, 'b-x');
plotevent(t, indx{E_ASPMT}, 'S', 8, stat, 'b-*');
plotevent(t, indx{E_VP}, 'P', -9, stat, 'r-o');
plotevent(t, indx{E_VS}, 'S', -9, stat, 'b-v');
plotevent(t, indx{E_VSBLK}, 'S', -6, stat, 'b-.x');
plotevent(t, indx{E_VES}, 'S', -9, stat, 'b-p');
plotevent(t, indx{E_VRS}, 'S', -6, stat, 'b-x');

% optional plot of pacemaker timers and windows
% % plot windows 
% if ~isempty(iARP), plotbar(t(iARP), ARP(iARP), 1, 2, 'g-'); end;
% if ~isempty(iAB), plotbar(t(iAB), AB(iAB), 0, 1, 'k-'); end;
% if ~isempty(iFFP), plotbar(t(iFFP), FFP(iFFP), 2, 3, 'c-'); end;
% if ~isempty(iPVARP), plotbar(t(iPVARP), PVARP(iPVARP), 3, 4, 'b-'); end;
% if ~isempty(iVRP), plotbar(t(iVRP), VRP(iVRP), -1, -2, 'g-'); end;
% if ~isempty(iVB), plotbar(t(iVB), VB(iVB), 0, -1, 'k-'); end;
% if ~isempty(iSW), plotbar(t(iSW), SW(iSW), -2, -3, 'c-'); end;
% if ~isempty(iUTI), plotbar(t(iUTI), UTI(iUTI), -3, -4, 'y-'); end;
% % plot timers 
% if ~isempty(iAP1), plottimer(t(iAP1), t(iAP2), 0, 9*(t(iAP2)-t(iAP1))./abs(AESC(iAP1)), 'm-'); end;
% if ~isempty(iVP1), plottimer(t(iVP1), t(iVP2), 0, -9*(t(iVP2)-t(iVP1))./VESC(iVP1), 'm-'); end;
% 
% if ~isempty(iSWVS),                               % SW_Vs and SW_Vp
%     plotshift(t(iSWVS), t(iSWVS)+SW(iSWVS), -8, -8, 'm.', 'm:');
% end
% if ~isempty(iWKBa),                               % WKB at UTI
%     plotshift(t(iWKBa)+AVD(iWKBa), t(iWKBa)+VESC(iWKBa), -3, -4, 'm:', 'm:');
% end

text(-0.1, 0.5, 'AB', 'FontSize', 8, 'HorizontalAlignment', 'Right');
text(-0.1, 1.5, 'ARP', 'FontSize', 8, 'HorizontalAlignment', 'Right');
text(-0.1, 2.5, 'FF', 'FontSize', 8, 'HorizontalAlignment', 'Right');
text(-0.1, 3.5, 'PVARP', 'FontSize', 8, 'HorizontalAlignment', 'Right');
text(-0.1, -0.5, 'VB', 'FontSize', 8, 'HorizontalAlignment', 'Right');
text(-0.1, -1.5, 'VRP', 'FontSize', 8, 'HorizontalAlignment', 'Right');
text(-0.1, -2.5, 'SW', 'FontSize', 8, 'HorizontalAlignment', 'Right');
text(-0.1, -3.5, 'UTI', 'FontSize', 8, 'HorizontalAlignment', 'Right');

set(gca,'YTick',[]);
hold off

% plotevent: plot vector of verticle event markers and text labels
% t: time, indx: event indexes, flg: P/S flag, y: height, stat: event type/status
function plotevent(t, indx, flg, y, stat, LineSpec)
if ~isempty(indx),                               % not empty index vector
    stem(t(indx), y*ones(size(indx)), LineSpec); % plot verticle markers
    stat = stat(indx);                           % stat of specific events
    str1 = {}; str2 = {};                        % init empty strings
    if upper(flg) == 'P',                        % paced events   
        str1(find(stat > 0)) = {'C'};            % captured paces
        str1(find(stat < 0)) = {'L'};            % loss-of-capture paces
        str2 = num2str(abs(stat),'%.1f');        % pacing amplitude
    else                                         % sensed events
        str1(find(stat == 0)) = {'I'};           % intrinsic sense
        str1(find(stat == 1)) = {'E'};           % ectopic sense
        str1(find(stat == 2)) = {'N'};           % noise sense
        if y > 0,                                % atrial markers
            str1(find(stat == 3)) = {'R'};       % retrograde conducted As
        else,                                    % ventricle markers
            str1(find(stat == 3)) = {'S'};       % antegrade conducted Vs
        end
    end
    if y > 0,                                    % atrial markers
        y1 = y+1; y2 = y+2;                      % text y axis
    else                                         % ventricle markers
        y1 = y-1; y2 = y-2;                      % text y axis
    end
    if ~isempty(str1), 
        text(t(indx), y1*ones(size(indx)), str1, 'clipping', 'on'); 
    end
    if ~isempty(str2), 
        text(t(indx), y2*ones(size(indx)), str2, 'clipping', 'on', 'FontSize', 8); 
    end
end


% plotbar: plot vector of horizontal bars
% t: start point, L: bar length, y1: start y, y2: stop y
function plotbar(t, L, y1, y2, LineSpec)
ip = find(L>0);    % plot positive bar (active window) only
t = t(ip);
L = L(ip);
N = length(t);
x = [t, t, t+L, t+L, t]';
y = [y1*ones(1,N); y2*ones(2,N); y1*ones(2,N)];
plot(x,y,LineSpec);

% plottimer: plot running timers
% t1: start time, t2: end time, y1: start y, y2: stop y
function plottimer(t1, t2, y1, y2, LineSpec)
N = length(t1);
x = [t1, t2]';
y = [y1*ones(N,1), y2]';
plot(x,y,LineSpec);

% plotshift: plot a shifted timer
% t1: start time, t2: end time, y1: start y, y2: stop y
function plotshift(t1, t2, y1, y2, MarkSpec, LineSpec)
N = length(t1);
x = [t1, t2]';
y0 = (y1+y2)/2;
y = [y0*ones(N,1), y0*ones(N,1)]';
plot(x,y,LineSpec);              % mark shifted timer
x = [t1, t1]';
y = [y1*ones(N,1), y2*ones(N,1)]';
plot(x,y,MarkSpec);              % mark timer start point 

